// Copyright (C) 2023 WYATT GROUP
// Please see the AUTHORS file for details.
//
// This program is free software: you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation, either version 3 of the License, or
// any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program. If not, see <https://www.gnu.org/licenses/>.

import 'package:wyatt_architecture/wyatt_architecture.dart';
import 'package:wyatt_cloud_messaging_bloc_base/src/core/helpers/app_exception_helper.dart';
import 'package:wyatt_cloud_messaging_bloc_base/src/domain/data_sources/remote/cloud_messaging_remote_data_source.dart';
import 'package:wyatt_cloud_messaging_bloc_base/src/domain/entities/remote_notifications.dart';
import 'package:wyatt_cloud_messaging_bloc_base/src/domain/repositories/cloud_messaging_repository.dart';
import 'package:wyatt_type_utils/wyatt_type_utils.dart';

class CloudMessagingRepositoryImpl extends CloudMessagingRepository {
  CloudMessagingRepositoryImpl({
    required CloudMessagingRemoteDataSource notificationRemoteDataSource,
  }) : _notificationRemoteDataSource = notificationRemoteDataSource;
  final CloudMessagingRemoteDataSource _notificationRemoteDataSource;

  @override
  FutureOrResult<String?> getToken() => Result.tryCatchAsync(
        _notificationRemoteDataSource.getToken,
        AppExceptionHelper.handleError,
      );

  @override
  FutureOrResult<void> subscribeToTopic(String topic) => Result.tryCatchAsync(
        () => _notificationRemoteDataSource.subscribeToTopic(topic),
        AppExceptionHelper.handleError,
      );

  @override
  FutureOrResult<void> unsubscribeFromTopic(String topic) =>
      Result.tryCatchAsync(
        () => _notificationRemoteDataSource.unsubscribeFromTopic(topic),
        AppExceptionHelper.handleError,
      );

  @override
  FutureOrResult<void> requestPermissions() => Result.tryCatchAsync(
        _notificationRemoteDataSource.requestPermissions,
        AppExceptionHelper.handleError,
      );

  @override
  FutureOrResult<Stream<RemoteNotification>> onNotification() =>
      Result.tryCatchAsync(
        _notificationRemoteDataSource.onNotification,
        AppExceptionHelper.handleError,
      );

  @override
  FutureOrResult<Stream<RemoteNotification>> onNotificationBackground() =>
      Result.tryCatchAsync(
        _notificationRemoteDataSource.onNotificationBackground,
        AppExceptionHelper.handleError,
      );

  @override
  FutureOrResult<Stream<RemoteNotification>> onNotificationForeground() =>
      Result.tryCatchAsync(
        _notificationRemoteDataSource.onNotificationForeground,
        AppExceptionHelper.handleError,
      );

  @override
  FutureOrResult<void> init() => Result.tryCatchAsync(
        _notificationRemoteDataSource.init,
        AppExceptionHelper.handleError,
      );
}
