// Copyright (C) 2023 WYATT GROUP
// Please see the AUTHORS file for details.
//
// This program is free software: you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation, either version 3 of the License, or
// any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program. If not, see <https://www.gnu.org/licenses/>.

import 'package:flutter/material.dart';
import 'package:wyatt_ui_components/wyatt_ui_components.dart';

/// {@template simple_icon_button_theme_extension}
/// Simple icon button theme extension that extends [ThemeExtension] and
/// implements copyWith, and lerp methods so you don't have to.
/// {@endtemplate}
class SimpleIconButtonThemeExtension
    extends ThemeExtension<SimpleIconButtonThemeExtension> {
  /// {@macro simple_icon_button_theme_extension}
  const SimpleIconButtonThemeExtension({
    this.disabledStyle,
    this.focusedStyle,
    this.hoveredStyle,
    this.normalStyle,
    this.tappedStyle,
  });

  /// Style of this button in disabled state
  final SimpleIconButtonStyle? disabledStyle;

  /// Style of this button in focused state
  final SimpleIconButtonStyle? focusedStyle;

  /// Style of this button in hovered state
  final SimpleIconButtonStyle? hoveredStyle;

  /// Style of this button in normal state
  final SimpleIconButtonStyle? normalStyle;

  /// Style of this button in tapped state
  final SimpleIconButtonStyle? tappedStyle;

  @override
  ThemeExtension<SimpleIconButtonThemeExtension> copyWith({
    SimpleIconButtonStyle? disabledStyle,
    SimpleIconButtonStyle? focusedStyle,
    SimpleIconButtonStyle? hoveredStyle,
    SimpleIconButtonStyle? normalStyle,
    SimpleIconButtonStyle? tappedStyle,
  }) =>
      SimpleIconButtonThemeExtension(
        disabledStyle: disabledStyle ?? this.disabledStyle,
        focusedStyle: focusedStyle ?? this.focusedStyle,
        hoveredStyle: hoveredStyle ?? this.hoveredStyle,
        normalStyle: normalStyle ?? this.normalStyle,
        tappedStyle: tappedStyle ?? this.tappedStyle,
      );

  @override
  ThemeExtension<SimpleIconButtonThemeExtension> lerp(
    covariant ThemeExtension<SimpleIconButtonThemeExtension>? other,
    double t,
  ) {
    if (other is! SimpleIconButtonThemeExtension) {
      return this;
    }
    return SimpleIconButtonThemeExtension(
      disabledStyle:
          SimpleIconButtonStyle.lerp(disabledStyle, other.disabledStyle, t),
      focusedStyle:
          SimpleIconButtonStyle.lerp(focusedStyle, other.focusedStyle, t),
      hoveredStyle:
          SimpleIconButtonStyle.lerp(hoveredStyle, other.hoveredStyle, t),
      normalStyle:
          SimpleIconButtonStyle.lerp(normalStyle, other.normalStyle, t),
      tappedStyle:
          SimpleIconButtonStyle.lerp(tappedStyle, other.tappedStyle, t),
    );
  }
}
